{% load dump_die %}


<a
  class="arrow-toggle {{ collapsable.function.class }}"
  title="[Ctrl+click] Expand all children"
  {% if collapsable.attribute.always_show == 'false' %}
    data-toggle="collapse"
  {% endif %}
  data-target=".{{ unique }}{{ root_count }}-functions"
  data-dd-type="func"
  aria-label="Open/Close"
  aria-expanded="{{ collapsable.function.aria }}"
>
  <span class="section_name">Functions</span>
  <span id="arrow-{{ unique }}{{ root_count }}-functions" class="arrow">
    {{ collapsable.function.arrow }}
  </span>
</a>

<div
  class="li-wrapper collapse {{ unique }}{{ root_count }}-functions {{ collapsable.function.show }}"
  data-unique-functions="{{ unique }}{{ root_count }}-functions"
>
  {% for attr, params, doc, access_modifier in functions %}
    <li class="function-li">
      {% if access_modifier %}
        <span class="access-modifier">{{ access_modifier }}</span>
      {% endif %}

      <span class="function" title="Function">
        {{ attr }}<span class="braces">(</span><span class="params">{{ params }}</span><span class="braces">)</span>
      </span>:
      {% if doc %}
        {# Handle function doc output. #}
        {% if multiline_function_docs %}
          <span class="docs indent-docs">{{ doc|linebreaks }}</span>
        {% else %}
          <span class="docs">{{ doc }}</span>
        {% endif  %}
      {% else %}
        <span class="empty">No Documentation</span>
      {% endif %}
    </li>
  {% empty %}
    <span class="empty" title="No Functions">No Functions</span>
  {% endfor %}
</div>
