{% load static %}
<!-- Required Libraries -->
<link rel="shortcut icon" href="data:image/x-icon;," type="image/x-icon"> {# Prevent chrome from requesting page again #}
<link rel="stylesheet" href="https://maxcdn.bootstrapcdn.com/bootstrap/4.0.0/css/bootstrap.min.css" integrity="sha384-Gn5384xqQ1aoWXA+058RXPxPg6fy4IWvTNh0E263XmFcJlSAwiGgFAW/dAiS6JXm" crossorigin="anonymous">
<!-- DD styles -->
<link rel="stylesheet" href="{% static 'django_dump_die/css/dd.css' %}">